<html>
  <head>
  <h1>Submission Proxy</h1>
  <div>Policy applied: {{.PolicyName}}</div>
  <div>Log list extracted from address: {{.LogListPath}}</div>
  <div>Log list:
  	<div style= "overflow:auto">{{.LogListJSON}}</div>
  </div>
 </html>