# frozen_string_literal: true
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: rekor/v2/hashedrekord.proto

require 'google/protobuf'

require 'google/api/field_behavior_pb'
require 'sigstore_common_pb'
require 'rekor/v2/verifier_pb'


descriptor_data = "\n\x1brekor/v2/hashedrekord.proto\x12\x15\x64\x65v.sigstore.rekor.v2\x1a\x1fgoogle/api/field_behavior.proto\x1a\x15sigstore_common.proto\x1a\x17rekor/v2/verifier.proto\"h\n\x17HashedRekordRequestV002\x12\x13\n\x06\x64igest\x18\x01 \x01(\x0c\x42\x03\xe0\x41\x02\x12\x38\n\tsignature\x18\x02 \x01(\x0b\x32 .dev.sigstore.rekor.v2.SignatureB\x03\xe0\x41\x02\"\x8b\x01\n\x18HashedRekordLogEntryV002\x12\x35\n\x04\x64\x61ta\x18\x01 \x01(\x0b\x32\".dev.sigstore.common.v1.HashOutputB\x03\xe0\x41\x02\x12\x38\n\tsignature\x18\x02 \x01(\x0b\x32 .dev.sigstore.rekor.v2.SignatureB\x03\xe0\x41\x02\x42\x81\x01\n\x1b\x64\x65v.sigstore.proto.rekor.v2B\x13RekorV2HashedRekordP\x01Z5github.com/sigstore/protobuf-specs/gen/pb-go/rekor/v2\xea\x02\x13Sigstore::Rekor::V2b\x06proto3"

pool = ::Google::Protobuf::DescriptorPool.generated_pool
pool.add_serialized_file(descriptor_data)

module Sigstore
  module Rekor
    module V2
      HashedRekordRequestV002 = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("dev.sigstore.rekor.v2.HashedRekordRequestV002").msgclass
      HashedRekordLogEntryV002 = ::Google::Protobuf::DescriptorPool.generated_pool.lookup("dev.sigstore.rekor.v2.HashedRekordLogEntryV002").msgclass
    end
  end
end
