// Copyright 2026 Google LLC All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

//go:build bench_memory

package daemon

import (
	"testing"

	"github.com/google/go-containerregistry/pkg/name"
)

const benchImage = "elasticsearch:9.0.3"

func loadAndAccess(t *testing.T, opts ...Option) {
	t.Helper()
	ref, err := name.ParseReference(benchImage)
	if err != nil {
		t.Fatalf("ParseReference: %v", err)
	}

	img, err := Image(ref, opts...)
	if err != nil {
		t.Fatalf("Image: %v", err)
	}

	// Access layers to trigger the docker save and buffering.
	layers, err := img.Layers()
	if err != nil {
		t.Fatalf("Layers: %v", err)
	}
	t.Logf("loaded %d layers", len(layers))
}

func TestBenchMemoryBacked(t *testing.T) {
	loadAndAccess(t, WithBufferedOpener())
}

func TestBenchFileBacked(t *testing.T) {
	loadAndAccess(t, WithFileBufferedOpener())
}
