syntax = "proto3";

// Import another proto file that doesn't have a package name

import "no_package_name/no_package_name.proto"; // import is relative to protoc's option --proto_path

option go_package = "github.com/twitchtv/twirp/internal/twirptest/no_package_name_importer";

service Svc2 {
  rpc Method(Msg) returns (Msg);
}
