// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

package validate

import (
	"testing"
)

// This test must be synchronized to avoid issues with -race
// NOTE(TEST).
func TestOptions_SetContinueOnErrors(_ *testing.T) {
}
