// Copyright 2018 Google LLC All Rights Reserved.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//    http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package authn

import (
	"testing"
)

func TestBearer(t *testing.T) {
	anon := &Bearer{Token: "bazinga"}

	auth, err := anon.Authorization()
	if err != nil {
		t.Errorf("Authorization() = %v", err)
	}
	if got, want := auth.RegistryToken, "bazinga"; got != want {
		t.Errorf("Authorization(); got %v, want %v", got, want)
	}
}
