// Code generated by go-swagger; DO NOT EDIT.

// Copyright 2022 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.
//

package timestamp

// This file was generated by the swagger tool.
// Editing this file might prove futile when you re-run the generate command

import (
	"net/http"

	"github.com/go-openapi/runtime/middleware"
)

// GetTimestampResponseHandlerFunc turns a function with the right signature into a get timestamp response handler
type GetTimestampResponseHandlerFunc func(GetTimestampResponseParams) middleware.Responder

// Handle executing the request and returning a response
func (fn GetTimestampResponseHandlerFunc) Handle(params GetTimestampResponseParams) middleware.Responder {
	return fn(params)
}

// GetTimestampResponseHandler interface for that can handle valid get timestamp response params
type GetTimestampResponseHandler interface {
	Handle(GetTimestampResponseParams) middleware.Responder
}

// NewGetTimestampResponse creates a new http.Handler for the get timestamp response operation
func NewGetTimestampResponse(ctx *middleware.Context, handler GetTimestampResponseHandler) *GetTimestampResponse {
	return &GetTimestampResponse{Context: ctx, Handler: handler}
}

/*
	GetTimestampResponse swagger:route POST /api/v1/timestamp timestamp getTimestampResponse

Generates a new timestamp response and creates a new log entry for the timestamp in the transparency log
*/
type GetTimestampResponse struct {
	Context *middleware.Context
	Handler GetTimestampResponseHandler
}

func (o *GetTimestampResponse) ServeHTTP(rw http.ResponseWriter, r *http.Request) {
	route, rCtx, _ := o.Context.RouteInfo(r)
	if rCtx != nil {
		*r = *rCtx
	}
	var Params = NewGetTimestampResponseParams()
	if err := o.Context.BindValidRequest(r, route, &Params); err != nil { // bind params
		o.Context.Respond(rw, r, route.Produces, route, err)
		return
	}

	res := o.Handler.Handle(Params) // actually handle the request

	o.Context.Respond(rw, r, route.Produces, route, res)

}
