/*
 * A Bit of Everything
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 1.0
 * Contact: none@example.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package abe
// NestedDeepEnum : DeepEnum is one or zero.   - FALSE: FALSE is false.  - TRUE: TRUE is true.
type NestedDeepEnum string

// List of NestedDeepEnum
const (
	FALSE_NestedDeepEnum NestedDeepEnum = "FALSE"
	TRUE_NestedDeepEnum NestedDeepEnum = "TRUE"
)
