syntax = "proto3";

// Test that protoc-gen-twirp follows the same behavior as protoc-gen-go
// for converting RPCs and message names from snake case to camel case.
package twirp.internal.twirptest.snake_case_names;
option go_package = "/;snake_case_names";

message MakeHatArgs_v1 {
    message Hat_v1 {
      int32 size = 1;
      string color = 2;
      string name = 3;
    }

    message Size_v1 {
      int32 inches = 1;
    }
}

// A Haberdasher makes hats for clients.
service Haberdasher_v1 {
  rpc MakeHat_v1 (MakeHatArgs_v1.Size_v1) returns (MakeHatArgs_v1.Hat_v1);
}
