syntax = "proto3";

// Test to make sure that importing other packages doesnt break
package twirp.internal.twirptest.importable;
option go_package = "github.com/twitchtv/twirp/internal/twirptest/importable";

message Msg {}

service Svc {
  rpc Send(Msg) returns(Msg);
}
