// SPDX-FileCopyrightText: Copyright 2015-2025 go-swagger maintainers
// SPDX-License-Identifier: Apache-2.0

// Package runtime exposes runtime client and server components
// for go-openapi toolkit.
package runtime
