// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v4.24.4
// source: in_toto_attestation/predicates/test_result/v0/test_result.proto

package v0

import (
	v1 "github.com/in-toto/attestation/go/v1"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

type TestResult struct {
	state         protoimpl.MessageState   `protogen:"open.v1"`
	Result        string                   `protobuf:"bytes,1,opt,name=result,proto3" json:"result,omitempty"`
	Configuration []*v1.ResourceDescriptor `protobuf:"bytes,2,rep,name=configuration,proto3" json:"configuration,omitempty"`
	Url           string                   `protobuf:"bytes,3,opt,name=url,proto3" json:"url,omitempty"`
	PassedTests   []string                 `protobuf:"bytes,4,rep,name=passed_tests,json=passedTests,proto3" json:"passed_tests,omitempty"`
	WarnedTests   []string                 `protobuf:"bytes,5,rep,name=warned_tests,json=warnedTests,proto3" json:"warned_tests,omitempty"`
	FailedTests   []string                 `protobuf:"bytes,6,rep,name=failed_tests,json=failedTests,proto3" json:"failed_tests,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *TestResult) Reset() {
	*x = TestResult{}
	mi := &file_in_toto_attestation_predicates_test_result_v0_test_result_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *TestResult) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*TestResult) ProtoMessage() {}

func (x *TestResult) ProtoReflect() protoreflect.Message {
	mi := &file_in_toto_attestation_predicates_test_result_v0_test_result_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use TestResult.ProtoReflect.Descriptor instead.
func (*TestResult) Descriptor() ([]byte, []int) {
	return file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescGZIP(), []int{0}
}

func (x *TestResult) GetResult() string {
	if x != nil {
		return x.Result
	}
	return ""
}

func (x *TestResult) GetConfiguration() []*v1.ResourceDescriptor {
	if x != nil {
		return x.Configuration
	}
	return nil
}

func (x *TestResult) GetUrl() string {
	if x != nil {
		return x.Url
	}
	return ""
}

func (x *TestResult) GetPassedTests() []string {
	if x != nil {
		return x.PassedTests
	}
	return nil
}

func (x *TestResult) GetWarnedTests() []string {
	if x != nil {
		return x.WarnedTests
	}
	return nil
}

func (x *TestResult) GetFailedTests() []string {
	if x != nil {
		return x.FailedTests
	}
	return nil
}

var File_in_toto_attestation_predicates_test_result_v0_test_result_proto protoreflect.FileDescriptor

const file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDesc = "" +
	"\n" +
	"?in_toto_attestation/predicates/test_result/v0/test_result.proto\x12-in_toto_attestation.predicates.test_result.v0\x1a0in_toto_attestation/v1/resource_descriptor.proto\"\xf1\x01\n" +
	"\n" +
	"TestResult\x12\x16\n" +
	"\x06result\x18\x01 \x01(\tR\x06result\x12P\n" +
	"\rconfiguration\x18\x02 \x03(\v2*.in_toto_attestation.v1.ResourceDescriptorR\rconfiguration\x12\x10\n" +
	"\x03url\x18\x03 \x01(\tR\x03url\x12!\n" +
	"\fpassed_tests\x18\x04 \x03(\tR\vpassedTests\x12!\n" +
	"\fwarned_tests\x18\x05 \x03(\tR\vwarnedTests\x12!\n" +
	"\ffailed_tests\x18\x06 \x03(\tR\vfailedTestsBu\n" +
	"6io.github.intoto.attestation.predicates.test_result.v0Z;github.com/in-toto/attestation/go/predicates/test_result/v0b\x06proto3"

var (
	file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescOnce sync.Once
	file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescData []byte
)

func file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescGZIP() []byte {
	file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescOnce.Do(func() {
		file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDesc), len(file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDesc)))
	})
	return file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDescData
}

var file_in_toto_attestation_predicates_test_result_v0_test_result_proto_msgTypes = make([]protoimpl.MessageInfo, 1)
var file_in_toto_attestation_predicates_test_result_v0_test_result_proto_goTypes = []any{
	(*TestResult)(nil),            // 0: in_toto_attestation.predicates.test_result.v0.TestResult
	(*v1.ResourceDescriptor)(nil), // 1: in_toto_attestation.v1.ResourceDescriptor
}
var file_in_toto_attestation_predicates_test_result_v0_test_result_proto_depIdxs = []int32{
	1, // 0: in_toto_attestation.predicates.test_result.v0.TestResult.configuration:type_name -> in_toto_attestation.v1.ResourceDescriptor
	1, // [1:1] is the sub-list for method output_type
	1, // [1:1] is the sub-list for method input_type
	1, // [1:1] is the sub-list for extension type_name
	1, // [1:1] is the sub-list for extension extendee
	0, // [0:1] is the sub-list for field type_name
}

func init() { file_in_toto_attestation_predicates_test_result_v0_test_result_proto_init() }
func file_in_toto_attestation_predicates_test_result_v0_test_result_proto_init() {
	if File_in_toto_attestation_predicates_test_result_v0_test_result_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDesc), len(file_in_toto_attestation_predicates_test_result_v0_test_result_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   1,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_in_toto_attestation_predicates_test_result_v0_test_result_proto_goTypes,
		DependencyIndexes: file_in_toto_attestation_predicates_test_result_v0_test_result_proto_depIdxs,
		MessageInfos:      file_in_toto_attestation_predicates_test_result_v0_test_result_proto_msgTypes,
	}.Build()
	File_in_toto_attestation_predicates_test_result_v0_test_result_proto = out.File
	file_in_toto_attestation_predicates_test_result_v0_test_result_proto_goTypes = nil
	file_in_toto_attestation_predicates_test_result_v0_test_result_proto_depIdxs = nil
}
