//
// Copyright 2022 The Sigstore Authors.
//
// Licensed under the Apache License, Version 2.0 (the "License");
// you may not use this file except in compliance with the License.
// You may obtain a copy of the License at
//
//     http://www.apache.org/licenses/LICENSE-2.0
//
// Unless required by applicable law or agreed to in writing, software
// distributed under the License is distributed on an "AS IS" BASIS,
// WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
// See the License for the specific language governing permissions and
// limitations under the License.

package app

import (
	"testing"
)

func TestValidateTimestampServerURL(t *testing.T) {
	type test struct {
		caseDesc        string
		timestampServer string
		expectSuccess   bool
	}

	tests := []test{
		{
			caseDesc:      "value not specified",
			expectSuccess: false,
		},
		{
			caseDesc:        "valid timestamp_server value",
			timestampServer: "http://localhost:3000",
			expectSuccess:   true,
		},
		{
			caseDesc:        "valid URL, invalid scheme",
			timestampServer: "ldap://localhost:3000",
			expectSuccess:   false,
		},
		{
			caseDesc:        "invalid URL",
			timestampServer: "hteeteepeeColonSlashSlashlocalhost:3000",
			expectSuccess:   false,
		},
		{
			caseDesc:        "local path",
			timestampServer: "/localhost",
			expectSuccess:   false,
		},
	}

	for _, tc := range tests {
		if err := rootCmd.PersistentFlags().Set("timestamp_server", tc.timestampServer); (err == nil) != tc.expectSuccess {
			t.Errorf("unexpected result in '%v': %v", tc.caseDesc, err)
		}
	}
}
