// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.36.11
// 	protoc        v4.24.4
// source: in_toto_attestation/predicates/vsa/v0/vsa.proto

package v0

import (
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	timestamppb "google.golang.org/protobuf/types/known/timestamppb"
	reflect "reflect"
	sync "sync"
	unsafe "unsafe"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Proto representation of predicate type https://slsa.dev/verification_summary/v0.2
// Validation of all fields is left to the users of this proto.
type VerificationSummary struct {
	state              protoimpl.MessageState                  `protogen:"open.v1"`
	Verifier           *VerificationSummary_Verifier           `protobuf:"bytes,1,opt,name=verifier,proto3" json:"verifier,omitempty"`
	TimeVerified       *timestamppb.Timestamp                  `protobuf:"bytes,2,opt,name=time_verified,proto3" json:"time_verified,omitempty"`
	ResourceUri        string                                  `protobuf:"bytes,3,opt,name=resource_uri,proto3" json:"resource_uri,omitempty"`
	Policy             *VerificationSummary_Policy             `protobuf:"bytes,4,opt,name=policy,proto3" json:"policy,omitempty"`
	InputAttestations  []*VerificationSummary_InputAttestation `protobuf:"bytes,5,rep,name=input_attestations,proto3" json:"input_attestations,omitempty"`
	VerificationResult string                                  `protobuf:"bytes,6,opt,name=verification_result,proto3" json:"verification_result,omitempty"`
	PolicyLevel        string                                  `protobuf:"bytes,7,opt,name=policy_level,proto3" json:"policy_level,omitempty"`
	DependencyLevels   map[string]uint64                       `protobuf:"bytes,8,rep,name=dependency_levels,proto3" json:"dependency_levels,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"varint,2,opt,name=value"`
	unknownFields      protoimpl.UnknownFields
	sizeCache          protoimpl.SizeCache
}

func (x *VerificationSummary) Reset() {
	*x = VerificationSummary{}
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VerificationSummary) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VerificationSummary) ProtoMessage() {}

func (x *VerificationSummary) ProtoReflect() protoreflect.Message {
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VerificationSummary.ProtoReflect.Descriptor instead.
func (*VerificationSummary) Descriptor() ([]byte, []int) {
	return file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescGZIP(), []int{0}
}

func (x *VerificationSummary) GetVerifier() *VerificationSummary_Verifier {
	if x != nil {
		return x.Verifier
	}
	return nil
}

func (x *VerificationSummary) GetTimeVerified() *timestamppb.Timestamp {
	if x != nil {
		return x.TimeVerified
	}
	return nil
}

func (x *VerificationSummary) GetResourceUri() string {
	if x != nil {
		return x.ResourceUri
	}
	return ""
}

func (x *VerificationSummary) GetPolicy() *VerificationSummary_Policy {
	if x != nil {
		return x.Policy
	}
	return nil
}

func (x *VerificationSummary) GetInputAttestations() []*VerificationSummary_InputAttestation {
	if x != nil {
		return x.InputAttestations
	}
	return nil
}

func (x *VerificationSummary) GetVerificationResult() string {
	if x != nil {
		return x.VerificationResult
	}
	return ""
}

func (x *VerificationSummary) GetPolicyLevel() string {
	if x != nil {
		return x.PolicyLevel
	}
	return ""
}

func (x *VerificationSummary) GetDependencyLevels() map[string]uint64 {
	if x != nil {
		return x.DependencyLevels
	}
	return nil
}

type VerificationSummary_Verifier struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Id            string                 `protobuf:"bytes,1,opt,name=id,proto3" json:"id,omitempty"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VerificationSummary_Verifier) Reset() {
	*x = VerificationSummary_Verifier{}
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VerificationSummary_Verifier) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VerificationSummary_Verifier) ProtoMessage() {}

func (x *VerificationSummary_Verifier) ProtoReflect() protoreflect.Message {
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VerificationSummary_Verifier.ProtoReflect.Descriptor instead.
func (*VerificationSummary_Verifier) Descriptor() ([]byte, []int) {
	return file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescGZIP(), []int{0, 0}
}

func (x *VerificationSummary_Verifier) GetId() string {
	if x != nil {
		return x.Id
	}
	return ""
}

type VerificationSummary_Policy struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Uri           string                 `protobuf:"bytes,1,opt,name=uri,proto3" json:"uri,omitempty"`
	Digest        map[string]string      `protobuf:"bytes,2,rep,name=digest,proto3" json:"digest,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VerificationSummary_Policy) Reset() {
	*x = VerificationSummary_Policy{}
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[2]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VerificationSummary_Policy) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VerificationSummary_Policy) ProtoMessage() {}

func (x *VerificationSummary_Policy) ProtoReflect() protoreflect.Message {
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[2]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VerificationSummary_Policy.ProtoReflect.Descriptor instead.
func (*VerificationSummary_Policy) Descriptor() ([]byte, []int) {
	return file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescGZIP(), []int{0, 1}
}

func (x *VerificationSummary_Policy) GetUri() string {
	if x != nil {
		return x.Uri
	}
	return ""
}

func (x *VerificationSummary_Policy) GetDigest() map[string]string {
	if x != nil {
		return x.Digest
	}
	return nil
}

type VerificationSummary_InputAttestation struct {
	state         protoimpl.MessageState `protogen:"open.v1"`
	Uri           string                 `protobuf:"bytes,1,opt,name=uri,proto3" json:"uri,omitempty"`
	Digest        map[string]string      `protobuf:"bytes,2,rep,name=digest,proto3" json:"digest,omitempty" protobuf_key:"bytes,1,opt,name=key" protobuf_val:"bytes,2,opt,name=value"`
	unknownFields protoimpl.UnknownFields
	sizeCache     protoimpl.SizeCache
}

func (x *VerificationSummary_InputAttestation) Reset() {
	*x = VerificationSummary_InputAttestation{}
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[3]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *VerificationSummary_InputAttestation) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*VerificationSummary_InputAttestation) ProtoMessage() {}

func (x *VerificationSummary_InputAttestation) ProtoReflect() protoreflect.Message {
	mi := &file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes[3]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use VerificationSummary_InputAttestation.ProtoReflect.Descriptor instead.
func (*VerificationSummary_InputAttestation) Descriptor() ([]byte, []int) {
	return file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescGZIP(), []int{0, 2}
}

func (x *VerificationSummary_InputAttestation) GetUri() string {
	if x != nil {
		return x.Uri
	}
	return ""
}

func (x *VerificationSummary_InputAttestation) GetDigest() map[string]string {
	if x != nil {
		return x.Digest
	}
	return nil
}

var File_in_toto_attestation_predicates_vsa_v0_vsa_proto protoreflect.FileDescriptor

const file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDesc = "" +
	"\n" +
	"/in_toto_attestation/predicates/vsa/v0/vsa.proto\x12%in_toto_attestation.predicates.vsa.v0\x1a\x1fgoogle/protobuf/timestamp.proto\"\xfd\b\n" +
	"\x13VerificationSummary\x12_\n" +
	"\bverifier\x18\x01 \x01(\v2C.in_toto_attestation.predicates.vsa.v0.VerificationSummary.VerifierR\bverifier\x12@\n" +
	"\rtime_verified\x18\x02 \x01(\v2\x1a.google.protobuf.TimestampR\rtime_verified\x12\"\n" +
	"\fresource_uri\x18\x03 \x01(\tR\fresource_uri\x12Y\n" +
	"\x06policy\x18\x04 \x01(\v2A.in_toto_attestation.predicates.vsa.v0.VerificationSummary.PolicyR\x06policy\x12{\n" +
	"\x12input_attestations\x18\x05 \x03(\v2K.in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestationR\x12input_attestations\x120\n" +
	"\x13verification_result\x18\x06 \x01(\tR\x13verification_result\x12\"\n" +
	"\fpolicy_level\x18\a \x01(\tR\fpolicy_level\x12~\n" +
	"\x11dependency_levels\x18\b \x03(\v2P.in_toto_attestation.predicates.vsa.v0.VerificationSummary.DependencyLevelsEntryR\x11dependency_levels\x1a\x1a\n" +
	"\bVerifier\x12\x0e\n" +
	"\x02id\x18\x01 \x01(\tR\x02id\x1a\xbc\x01\n" +
	"\x06Policy\x12\x10\n" +
	"\x03uri\x18\x01 \x01(\tR\x03uri\x12e\n" +
	"\x06digest\x18\x02 \x03(\v2M.in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy.DigestEntryR\x06digest\x1a9\n" +
	"\vDigestEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\x1a\xd0\x01\n" +
	"\x10InputAttestation\x12\x10\n" +
	"\x03uri\x18\x01 \x01(\tR\x03uri\x12o\n" +
	"\x06digest\x18\x02 \x03(\v2W.in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation.DigestEntryR\x06digest\x1a9\n" +
	"\vDigestEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\tR\x05value:\x028\x01\x1aC\n" +
	"\x15DependencyLevelsEntry\x12\x10\n" +
	"\x03key\x18\x01 \x01(\tR\x03key\x12\x14\n" +
	"\x05value\x18\x02 \x01(\x04R\x05value:\x028\x01Be\n" +
	".io.github.intoto.attestation.predicates.vsa.v0Z3github.com/in-toto/attestation/go/predicates/vsa/v0b\x06proto3"

var (
	file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescOnce sync.Once
	file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescData []byte
)

func file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescGZIP() []byte {
	file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescOnce.Do(func() {
		file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescData = protoimpl.X.CompressGZIP(unsafe.Slice(unsafe.StringData(file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDesc), len(file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDesc)))
	})
	return file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDescData
}

var file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes = make([]protoimpl.MessageInfo, 7)
var file_in_toto_attestation_predicates_vsa_v0_vsa_proto_goTypes = []any{
	(*VerificationSummary)(nil),                  // 0: in_toto_attestation.predicates.vsa.v0.VerificationSummary
	(*VerificationSummary_Verifier)(nil),         // 1: in_toto_attestation.predicates.vsa.v0.VerificationSummary.Verifier
	(*VerificationSummary_Policy)(nil),           // 2: in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy
	(*VerificationSummary_InputAttestation)(nil), // 3: in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation
	nil,                           // 4: in_toto_attestation.predicates.vsa.v0.VerificationSummary.DependencyLevelsEntry
	nil,                           // 5: in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy.DigestEntry
	nil,                           // 6: in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation.DigestEntry
	(*timestamppb.Timestamp)(nil), // 7: google.protobuf.Timestamp
}
var file_in_toto_attestation_predicates_vsa_v0_vsa_proto_depIdxs = []int32{
	1, // 0: in_toto_attestation.predicates.vsa.v0.VerificationSummary.verifier:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.Verifier
	7, // 1: in_toto_attestation.predicates.vsa.v0.VerificationSummary.time_verified:type_name -> google.protobuf.Timestamp
	2, // 2: in_toto_attestation.predicates.vsa.v0.VerificationSummary.policy:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy
	3, // 3: in_toto_attestation.predicates.vsa.v0.VerificationSummary.input_attestations:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation
	4, // 4: in_toto_attestation.predicates.vsa.v0.VerificationSummary.dependency_levels:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.DependencyLevelsEntry
	5, // 5: in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy.digest:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.Policy.DigestEntry
	6, // 6: in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation.digest:type_name -> in_toto_attestation.predicates.vsa.v0.VerificationSummary.InputAttestation.DigestEntry
	7, // [7:7] is the sub-list for method output_type
	7, // [7:7] is the sub-list for method input_type
	7, // [7:7] is the sub-list for extension type_name
	7, // [7:7] is the sub-list for extension extendee
	0, // [0:7] is the sub-list for field type_name
}

func init() { file_in_toto_attestation_predicates_vsa_v0_vsa_proto_init() }
func file_in_toto_attestation_predicates_vsa_v0_vsa_proto_init() {
	if File_in_toto_attestation_predicates_vsa_v0_vsa_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: unsafe.Slice(unsafe.StringData(file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDesc), len(file_in_toto_attestation_predicates_vsa_v0_vsa_proto_rawDesc)),
			NumEnums:      0,
			NumMessages:   7,
			NumExtensions: 0,
			NumServices:   0,
		},
		GoTypes:           file_in_toto_attestation_predicates_vsa_v0_vsa_proto_goTypes,
		DependencyIndexes: file_in_toto_attestation_predicates_vsa_v0_vsa_proto_depIdxs,
		MessageInfos:      file_in_toto_attestation_predicates_vsa_v0_vsa_proto_msgTypes,
	}.Build()
	File_in_toto_attestation_predicates_vsa_v0_vsa_proto = out.File
	file_in_toto_attestation_predicates_vsa_v0_vsa_proto_goTypes = nil
	file_in_toto_attestation_predicates_vsa_v0_vsa_proto_depIdxs = nil
}
