/* your custom css */

@media only screen and (min-device-width: 360px) and (max-device-width: 736px) {
}

@media only screen and (min-width: 1024px) {
}

@media only screen and (max-width: 1023px) {
}

@media only screen and (min-width: 1400px) {
}

@media only screen and (min-width: 1500px) {
}

.mainContainer .wrapper p,
.blockElement p,
.mainContainer .wrapper ul li,
.mainContainer .wrapper ol li {
    line-height: 1.5;
}
