/*
 * A Bit of Everything
 *
 * No description provided (generated by Swagger Codegen https://github.com/swagger-api/swagger-codegen)
 *
 * API version: 1.0
 * Contact: none@example.com
 * Generated by: Swagger Codegen (https://github.com/swagger-api/swagger-codegen.git)
 */

package abe

type UpdateV2RequestRequestForUpdateIncludesTheMessageAndTheUpdateMask struct {
	Abe *ABitOfEverything4 `json:"abe,omitempty"`
	// The paths to update.
	UpdateMask string `json:"updateMask,omitempty"`
}
