// Code generated by protoc-gen-go. DO NOT EDIT.
// versions:
// 	protoc-gen-go v1.35.1
// 	protoc        (unknown)
// source: examples/internal/proto/examplepb/remove_internal_comment.proto

package examplepb

import (
	_ "google.golang.org/genproto/googleapis/api/annotations"
	protoreflect "google.golang.org/protobuf/reflect/protoreflect"
	protoimpl "google.golang.org/protobuf/runtime/protoimpl"
	reflect "reflect"
	sync "sync"
)

const (
	// Verify that this generated code is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(20 - protoimpl.MinVersion)
	// Verify that runtime/protoimpl is sufficiently up-to-date.
	_ = protoimpl.EnforceVersion(protoimpl.MaxVersion - 20)
)

// Foo2Request (-- This comment should be excluded from OpenAPI output --)
type Foo2Request struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields

	// Username.
	// (-- This comment should be excluded
	// from OpenAPI output --)
	// Same row, single line break doesn't count on markdown.
	Username string `protobuf:"bytes,1,opt,name=username,proto3" json:"username,omitempty"`
	// Password.
	// (-- This comment should be excluded
	// from OpenAPI output --)
	//
	// New row.
	Password string `protobuf:"bytes,2,opt,name=password,proto3" json:"password,omitempty"`
}

func (x *Foo2Request) Reset() {
	*x = Foo2Request{}
	mi := &file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes[0]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Foo2Request) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Foo2Request) ProtoMessage() {}

func (x *Foo2Request) ProtoReflect() protoreflect.Message {
	mi := &file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes[0]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Foo2Request.ProtoReflect.Descriptor instead.
func (*Foo2Request) Descriptor() ([]byte, []int) {
	return file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescGZIP(), []int{0}
}

func (x *Foo2Request) GetUsername() string {
	if x != nil {
		return x.Username
	}
	return ""
}

func (x *Foo2Request) GetPassword() string {
	if x != nil {
		return x.Password
	}
	return ""
}

// (-- This comment should be excluded from OpenAPI output --)
type Foo2Reply struct {
	state         protoimpl.MessageState
	sizeCache     protoimpl.SizeCache
	unknownFields protoimpl.UnknownFields
}

func (x *Foo2Reply) Reset() {
	*x = Foo2Reply{}
	mi := &file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes[1]
	ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
	ms.StoreMessageInfo(mi)
}

func (x *Foo2Reply) String() string {
	return protoimpl.X.MessageStringOf(x)
}

func (*Foo2Reply) ProtoMessage() {}

func (x *Foo2Reply) ProtoReflect() protoreflect.Message {
	mi := &file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes[1]
	if x != nil {
		ms := protoimpl.X.MessageStateOf(protoimpl.Pointer(x))
		if ms.LoadMessageInfo() == nil {
			ms.StoreMessageInfo(mi)
		}
		return ms
	}
	return mi.MessageOf(x)
}

// Deprecated: Use Foo2Reply.ProtoReflect.Descriptor instead.
func (*Foo2Reply) Descriptor() ([]byte, []int) {
	return file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescGZIP(), []int{1}
}

var File_examples_internal_proto_examplepb_remove_internal_comment_proto protoreflect.FileDescriptor

var file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDesc = []byte{
	0x0a, 0x3f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x73, 0x2f, 0x69, 0x6e, 0x74, 0x65, 0x72,
	0x6e, 0x61, 0x6c, 0x2f, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c,
	0x65, 0x70, 0x62, 0x2f, 0x72, 0x65, 0x6d, 0x6f, 0x76, 0x65, 0x5f, 0x69, 0x6e, 0x74, 0x65, 0x72,
	0x6e, 0x61, 0x6c, 0x5f, 0x63, 0x6f, 0x6d, 0x6d, 0x65, 0x6e, 0x74, 0x2e, 0x70, 0x72, 0x6f, 0x74,
	0x6f, 0x12, 0x2e, 0x67, 0x72, 0x70, 0x63, 0x2e, 0x67, 0x61, 0x74, 0x65, 0x77, 0x61, 0x79, 0x2e,
	0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x73, 0x2e, 0x69, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x61,
	0x6c, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x70,
	0x62, 0x1a, 0x1c, 0x67, 0x6f, 0x6f, 0x67, 0x6c, 0x65, 0x2f, 0x61, 0x70, 0x69, 0x2f, 0x61, 0x6e,
	0x6e, 0x6f, 0x74, 0x61, 0x74, 0x69, 0x6f, 0x6e, 0x73, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x22,
	0x45, 0x0a, 0x0b, 0x46, 0x6f, 0x6f, 0x32, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x12, 0x1a,
	0x0a, 0x08, 0x75, 0x73, 0x65, 0x72, 0x6e, 0x61, 0x6d, 0x65, 0x18, 0x01, 0x20, 0x01, 0x28, 0x09,
	0x52, 0x08, 0x75, 0x73, 0x65, 0x72, 0x6e, 0x61, 0x6d, 0x65, 0x12, 0x1a, 0x0a, 0x08, 0x70, 0x61,
	0x73, 0x73, 0x77, 0x6f, 0x72, 0x64, 0x18, 0x02, 0x20, 0x01, 0x28, 0x09, 0x52, 0x08, 0x70, 0x61,
	0x73, 0x73, 0x77, 0x6f, 0x72, 0x64, 0x22, 0x0b, 0x0a, 0x09, 0x46, 0x6f, 0x6f, 0x32, 0x52, 0x65,
	0x70, 0x6c, 0x79, 0x32, 0xaa, 0x01, 0x0a, 0x0b, 0x46, 0x6f, 0x6f, 0x32, 0x53, 0x65, 0x72, 0x76,
	0x69, 0x63, 0x65, 0x12, 0x9a, 0x01, 0x0a, 0x04, 0x46, 0x6f, 0x6f, 0x32, 0x12, 0x3b, 0x2e, 0x67,
	0x72, 0x70, 0x63, 0x2e, 0x67, 0x61, 0x74, 0x65, 0x77, 0x61, 0x79, 0x2e, 0x65, 0x78, 0x61, 0x6d,
	0x70, 0x6c, 0x65, 0x73, 0x2e, 0x69, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x61, 0x6c, 0x2e, 0x70, 0x72,
	0x6f, 0x74, 0x6f, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x70, 0x62, 0x2e, 0x46, 0x6f,
	0x6f, 0x32, 0x52, 0x65, 0x71, 0x75, 0x65, 0x73, 0x74, 0x1a, 0x39, 0x2e, 0x67, 0x72, 0x70, 0x63,
	0x2e, 0x67, 0x61, 0x74, 0x65, 0x77, 0x61, 0x79, 0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65,
	0x73, 0x2e, 0x69, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x61, 0x6c, 0x2e, 0x70, 0x72, 0x6f, 0x74, 0x6f,
	0x2e, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x70, 0x62, 0x2e, 0x46, 0x6f, 0x6f, 0x32, 0x52,
	0x65, 0x70, 0x6c, 0x79, 0x22, 0x1a, 0x82, 0xd3, 0xe4, 0x93, 0x02, 0x14, 0x3a, 0x01, 0x2a, 0x22,
	0x0f, 0x2f, 0x76, 0x31, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x2f, 0x66, 0x6f, 0x6f,
	0x42, 0x4d, 0x5a, 0x4b, 0x67, 0x69, 0x74, 0x68, 0x75, 0x62, 0x2e, 0x63, 0x6f, 0x6d, 0x2f, 0x67,
	0x72, 0x70, 0x63, 0x2d, 0x65, 0x63, 0x6f, 0x73, 0x79, 0x73, 0x74, 0x65, 0x6d, 0x2f, 0x67, 0x72,
	0x70, 0x63, 0x2d, 0x67, 0x61, 0x74, 0x65, 0x77, 0x61, 0x79, 0x2f, 0x76, 0x32, 0x2f, 0x65, 0x78,
	0x61, 0x6d, 0x70, 0x6c, 0x65, 0x73, 0x2f, 0x69, 0x6e, 0x74, 0x65, 0x72, 0x6e, 0x61, 0x6c, 0x2f,
	0x70, 0x72, 0x6f, 0x74, 0x6f, 0x2f, 0x65, 0x78, 0x61, 0x6d, 0x70, 0x6c, 0x65, 0x70, 0x62, 0x62,
	0x06, 0x70, 0x72, 0x6f, 0x74, 0x6f, 0x33,
}

var (
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescOnce sync.Once
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescData = file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDesc
)

func file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescGZIP() []byte {
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescOnce.Do(func() {
		file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescData = protoimpl.X.CompressGZIP(file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescData)
	})
	return file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDescData
}

var file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes = make([]protoimpl.MessageInfo, 2)
var file_examples_internal_proto_examplepb_remove_internal_comment_proto_goTypes = []any{
	(*Foo2Request)(nil), // 0: grpc.gateway.examples.internal.proto.examplepb.Foo2Request
	(*Foo2Reply)(nil),   // 1: grpc.gateway.examples.internal.proto.examplepb.Foo2Reply
}
var file_examples_internal_proto_examplepb_remove_internal_comment_proto_depIdxs = []int32{
	0, // 0: grpc.gateway.examples.internal.proto.examplepb.Foo2Service.Foo2:input_type -> grpc.gateway.examples.internal.proto.examplepb.Foo2Request
	1, // 1: grpc.gateway.examples.internal.proto.examplepb.Foo2Service.Foo2:output_type -> grpc.gateway.examples.internal.proto.examplepb.Foo2Reply
	1, // [1:2] is the sub-list for method output_type
	0, // [0:1] is the sub-list for method input_type
	0, // [0:0] is the sub-list for extension type_name
	0, // [0:0] is the sub-list for extension extendee
	0, // [0:0] is the sub-list for field type_name
}

func init() { file_examples_internal_proto_examplepb_remove_internal_comment_proto_init() }
func file_examples_internal_proto_examplepb_remove_internal_comment_proto_init() {
	if File_examples_internal_proto_examplepb_remove_internal_comment_proto != nil {
		return
	}
	type x struct{}
	out := protoimpl.TypeBuilder{
		File: protoimpl.DescBuilder{
			GoPackagePath: reflect.TypeOf(x{}).PkgPath(),
			RawDescriptor: file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDesc,
			NumEnums:      0,
			NumMessages:   2,
			NumExtensions: 0,
			NumServices:   1,
		},
		GoTypes:           file_examples_internal_proto_examplepb_remove_internal_comment_proto_goTypes,
		DependencyIndexes: file_examples_internal_proto_examplepb_remove_internal_comment_proto_depIdxs,
		MessageInfos:      file_examples_internal_proto_examplepb_remove_internal_comment_proto_msgTypes,
	}.Build()
	File_examples_internal_proto_examplepb_remove_internal_comment_proto = out.File
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_rawDesc = nil
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_goTypes = nil
	file_examples_internal_proto_examplepb_remove_internal_comment_proto_depIdxs = nil
}
