# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: in_toto_attestation/predicates/release/v0/release.proto
"""Generated protocol buffer code."""

from google.protobuf import descriptor as _descriptor
from google.protobuf import descriptor_pool as _descriptor_pool
from google.protobuf import symbol_database as _symbol_database
from google.protobuf.internal import builder as _builder

# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


DESCRIPTOR = _descriptor_pool.Default().AddSerializedFile(
    b'\n7in_toto_attestation/predicates/release/v0/release.proto\x12)in_toto_attestation.predicates.release.v0"?\n\x07Release\x12\x0c\n\x04purl\x18\x01 \x01(\t\x12\x17\n\nrelease_id\x18\x02 \x01(\tH\x00\x88\x01\x01\x42\r\n\x0b_release_idBm\n2io.github.intoto.attestation.predicates.release.v0Z7github.com/in-toto/attestation/go/predicates/release/v0b\x06proto3'
)

_globals = globals()
_builder.BuildMessageAndEnumDescriptors(DESCRIPTOR, _globals)
_builder.BuildTopDescriptorsAndMessages(
    DESCRIPTOR, "in_toto_attestation.predicates.release.v0.release_pb2", _globals
)
if _descriptor._USE_C_DESCRIPTORS == False:
    _globals["DESCRIPTOR"]._options = None
    _globals[
        "DESCRIPTOR"
    ]._serialized_options = b"\n2io.github.intoto.attestation.predicates.release.v0Z7github.com/in-toto/attestation/go/predicates/release/v0"
    _globals["_RELEASE"]._serialized_start = 102
    _globals["_RELEASE"]._serialized_end = 165
# @@protoc_insertion_point(module_scope)
